]> Shamusworld >> Repos - rmac/blobdiff - rmac.c
Fix for bug #174 - when a macro contains an error, actually print the filename it...
[rmac] / rmac.c
diff --git a/rmac.c b/rmac.c
index c3da33dc734364f7516a3f819f8ea46a6f883a21..4f78759070ef6888b0c7897119e87e34ef396d9e 100644 (file)
--- a/rmac.c
+++ b/rmac.c
@@ -1,7 +1,7 @@
 //
// RMAC - Reboot's Macro Assembler for all Atari computers
 // RMAC.C - Main Application Code
-// Copyright (C) 199x Landon Dyer, 2011-2020 Reboot and Friends
+// Copyright (C) 199x Landon Dyer, 2011-2021 Reboot and Friends
 // RMAC derived from MADMAC v1.07 Written by Landon Dyer, 1986
 // Source utilised with the kind permission of Landon Dyer
 //
@@ -50,9 +50,10 @@ int endian;                                          // Host processor endianess (0 = LE, 1 = BE)
 char * objfname;                               // Object filename pointer
 char * firstfname;                             // First source filename
 char * cmdlnexec;                              // Executable name, pointer to ARGV[0]
-char searchpath[512] = { 0 };  // Search path for include files
+char searchpatha[512] = { 0 }; // Buffer to hold searchpath when specified
+char * searchpath = NULL;              // Search path for include files
 char defname[] = "noname.o";   // Default output filename
-int optim_flags[OPT_COUNT];            // Specific optimisations on/off matrix
+int optim_flags[OPT_COUNT_ALL];        // Specific optimisations on/off matrix
 int activecpu = CPU_68000;             // Active 68k CPU (68000 by default)
 int activefpu = FPU_NONE;              // Active FPU (none by default)
 int org68k_active = 0;                 // .org switch for 68k (only with RAW output format)
@@ -67,7 +68,6 @@ void strtoupper(char * s)
                *s++ &= 0xDF;
 }
 
-
 //
 // Manipulate file extension.
 //
@@ -100,6 +100,17 @@ char * fext(char * name, char * extension, int stripp)
        return name;
 }
 
+static int is_sep(char c)
+{
+    const char *seps = PATH_SEPS;
+
+    for (seps = PATH_SEPS; *seps; seps++) {
+        if (*seps == c)
+            return 1;
+    }
+
+    return 0;
+}
 
 //
 // Return 'item'nth element of semicolon-seperated pathnames specified in the
@@ -119,13 +130,13 @@ int nthpath(char * env_var, int itemno, char * buf)
                return 0;
 
        while (itemno--)
-               while (*s != EOS && *s++ != ';')
+               while (*s != EOS && !is_sep(*s++))
                        ;
 
        if (*s == EOS)
                return 0;
 
-       while (*s != EOS && *s != ';')
+       while (*s != EOS && !is_sep(*s))
                *buf++ = *s++;
 
        *buf++ = EOS;
@@ -133,7 +144,6 @@ int nthpath(char * env_var, int itemno, char * buf)
        return 1;
 }
 
-
 //
 // Display command line help
 //
@@ -173,7 +183,7 @@ void DisplayHelp(void)
                "                    o7: clr.l Dx to moveq #0,Dx                       (off)\n"
                "                    o8: adda.w/l #x,Dy to addq.w/l #x,Dy              (off)\n"
                "                    o9: adda.w/l #x,Dy to lea x(Dy),Dy                (off)\n"
-               "                    op: Enforce PC relative                           (off)\n"
+               "                    o10: Enforce PC relative (alternative: op)        (off)\n"
                "  ~o[value]         Turn a specific optimisation off\n"
                "  +oall             Turn all optimisations on\n"
                "  ~oall             Turn all optimisations off\n"
@@ -206,12 +216,11 @@ void DisplayVersion(void)
                "| '__| '_ ` _ \\ / _` |/ __|\n"
                "| |  | | | | | | (_| | (__ \n"
                "|_|  |_| |_| |_|\\__,_|\\___|\n"
-               "\nReboot's Macro Assembler\n"
-               "Copyright (C) 199x Landon Dyer, 2011-2020 Reboot\n"
+               "\nRenamed Macro Assembler\n"
+               "Copyright (C) 199x Landon Dyer, 2011-2021 Reboot and Friends\n"
                "V%01i.%01i.%01i %s (%s)\n\n", MAJOR, MINOR, PATCH, __DATE__, PLATFORM);
 }
 
-
 //
 // Parse optimisation options
 //
@@ -226,6 +235,9 @@ int ParseOptimization(char * optstring)
                else if (*optstring != '~')
                        return ERROR;
 
+        if (optstring[2] == 0)
+            return error(".opt called with zero arguments");
+
                if ((optstring[2] == 'a' || optstring[2] == 'A')
                        && (optstring[3] == 'l' || optstring[3] == 'L')
                        && (optstring[4] == 'l' || optstring[4] == 'L'))
@@ -272,7 +284,6 @@ int ParseOptimization(char * optstring)
        return OK;
 }
 
-
 //
 // Process command line arguments and do an assembly
 //
@@ -410,16 +421,18 @@ int Process(int argc, char ** argv)
                        case 'i':                               // Set directory search path
                        case 'I':
                        {
-                               strcat(searchpath, argv[argno] + 2);
-                strcat(searchpath, ";");
+                               strcat(searchpatha, argv[argno] + 2);
+                               strcat(searchpatha, ";");
+                               searchpath = searchpatha;
 
                                // Check to see if include paths actually exist
                                char current_path[256];
-                               for (i = current_path_index; nthpath("RMACPATH", i, current_path) != 0; i++)
+
+                               for(i=current_path_index; nthpath("RMACPATH", i, current_path)!=0; i++)
                                {
                                        if (strlen(current_path) > 0)
                                        {
-                                               DIR* test = opendir(current_path);
+                                               DIR * test = opendir(current_path);
 
                                                if (test == NULL)
                                                {
@@ -431,7 +444,8 @@ int Process(int argc, char ** argv)
                                                closedir(test);
                                        }
                                }
-                               current_path_index = i-1;
+
+                               current_path_index = i - 1;
                                break;
                        }
                        case 'l':                               // Produce listing file
@@ -741,7 +755,6 @@ int Process(int argc, char ** argv)
        return errcnt;
 }
 
-
 //
 // Determine processor endianess
 //
@@ -756,7 +769,6 @@ int GetEndianess(void)
        return 1;
 }
 
-
 //
 // Application entry point
 //
@@ -783,4 +795,3 @@ int main(int argc, char ** argv)
 
        return 0;
 }
-
@@ -1,7 +1,7 @@
 //
-// RMAC - Reboot's Macro Assembler for all Atari computers
+// RMAC - Renamed Macro Assembler for all Atari computers
 // RMAC.C - Main Application Code